@@ -6,7 +6,7 @@ use AnyEvent;
 use Bif::Client;
 use Coro;
 
-our $VERSION = '0.1.0_13';
+our $VERSION = '0.1.0_14';
 
 sub run {
     my $ctx = shift;
@@ -126,7 +126,15 @@ sub run {
                             values      => { merge => 1 },
                         );
 
+                        $client->on_update(
+                            sub {
+                                $ctx->lprint(
+                                    "$hub->{alias} [$pinfo->{path}]: $_[0]");
+                            }
+                        );
+
                         my $status = $client->export_project($pinfo);
+                        print "\n";
 
                         if ( $status eq 'ProjectExported' ) {
                             print "Project exported: $pinfo->{path}\n";
@@ -172,7 +180,7 @@ bif-export -  export a project to a remote hub
 
 =head1 VERSION
 
-0.1.0_13 (2014-04-23)
+0.1.0_14 (2014-04-24)
 
 =head1 SYNOPSIS
 
