@@ -6,7 +6,7 @@ use AnyEvent;
 use Bif::Client;
 use Coro;
 
-our $VERSION = '0.1.0_13';
+our $VERSION = '0.1.0_14';
 
 sub run {
     my $ctx = shift;
@@ -83,7 +83,15 @@ sub run {
                     );
 
                     foreach my $pinfo (@pinfo) {
+                        $client->on_update(
+                            sub {
+                                $ctx->lprint(
+                                    "$hub->{alias} [$pinfo->{path}]: $_[0]");
+                            }
+                        );
+
                         my $status = $client->import_project($pinfo);
+                        print "\n";
 
                         if ( $status eq 'ProjectImported' ) {
                             print "Project imported: $pinfo->{path}\n";
@@ -130,7 +138,7 @@ bif-import -  import projects from a remote hub
 
 =head1 VERSION
 
-0.1.0_13 (2014-04-23)
+0.1.0_14 (2014-04-24)
 
 =head1 SYNOPSIS
 
