@@ -12,7 +12,7 @@ use Log::Any::Plugin;
 use OptArgs;
 use Path::Tiny;
 
-our $VERSION = '0.1.0_13';
+our $VERSION = '0.1.0_14';
 
 arg directory => (
     isa      => 'Str',
@@ -68,7 +68,7 @@ sub run {
     );
 
     my $coro = async {
-        while ( $server->accept ) { }
+        $server->run;
         $server->disconnect;
         $cv->send;
     };
