@@ -1,11 +1,12 @@
 package Bif::Role::Sync::Repo;
 use strict;
 use warnings;
+use Coro;
 use DBIx::ThinSQL qw/qv/;
 use Log::Any '$log';
 use Role::Basic;
 
-our $VERSION = '0.1.0_13';
+our $VERSION = '0.1.0_14';
 
 my %import_functions = (
     NEW => {
@@ -37,34 +38,32 @@ sub real_import_repo {
     my $self = shift;
     my $db   = $self->db;
 
-    my ( $TOTAL, $total ) = $self->read;
+    my ( $action, $total ) = $self->read;
+    $total //= '*undef*';
 
-    if ( $TOTAL ne 'TOTAL' ) {
-        $self->write('ExpectedCount');
-        return 'ExpectedCount';
+    if ( $action eq 'QUIT' or $action eq 'INVALID' or $action eq 'EOF' ) {
+        return $action;
+    }
+    elsif ( $action ne 'TOTAL' or $total !~ m/^\d+$/ ) {
+        return "expected TOTAL <int> (not $action $total)";
     }
 
     my $ucount;
-    my $on_update = $self->on_update;
+    my $i   = $total;
+    my $got = 0;
 
-    while ( $total-- > 0 ) {
+    $self->updates_recv("$got/$total");
+    $self->trigger_on_update;
+
+    while ( $got < $total ) {
         my ( $action, $type, $ref ) = $self->read;
 
-        $on_update->( 'inbound updates: ' . $total ) if $on_update;
-
         if ( !exists $import_functions{$action} ) {
-            $self->write( 'BadMethod', $action );
-            return 'BadMethod';
-        }
-
-        if ( $action eq 'QUIT' or $action eq 'CANCEL' ) {
-            $self->write('QUIT');
-            return 'UnexpectedQuit';
+            return "not implemented: $action";
         }
 
         if ( !exists $import_functions{$action}->{$type} ) {
-            $self->write('NotImplemented');
-            return 'NotImplemented';
+            return "not implemented: $action $type";
         }
 
         if ( $action eq 'NEW' and $type eq 'update' ) {
@@ -86,12 +85,17 @@ sub real_import_repo {
                 values      => { merge => 1 },
             );
         }
+
+        $got++;
+        $self->updates_recv("$got/$total");
+        $self->trigger_on_update;
+
     }
 
-    $self->write('RepoImported');
-    my ($action) = $self->read;
-    return 'RepoImported' if $action eq 'RepoExported';
-    return $action;
+    $self->updates_recv($total);
+    $self->trigger_on_update;
+
+    return 'RepoImported';
 }
 
 sub real_sync_repo {
@@ -119,7 +123,7 @@ sub real_sync_repo {
     $self->write( 'MATCH', $prefix2, $here );
     my ( $action, $mprefix, $there ) = $self->read;
 
-    return 'ProtocolError'
+    return "expected MATCH $prefix2 {} (not $action $mprefix ...)"
       unless $action eq 'MATCH'
       and $mprefix eq $prefix2
       and ref $there eq 'HASH';
@@ -166,31 +170,33 @@ sub real_sync_repo {
 
     return unless $prefix eq '';
 
-    my ($total) = $self->db->xarray(
-        select => 'COALESCE(sum(t.ucount), 0)',
-        from   => "$tmp t",
-    );
+    my $send = async {
+        my ($total) = $self->db->xarray(
+            select => 'COALESCE(sum(t.ucount), 0)',
+            from   => "$tmp t",
+        );
 
-    $self->write( 'TOTAL', $total );
+        $self->write( 'TOTAL', $total );
 
-    my $update_list = $db->xprepare(
-        select => [
-            'u.id',                  'u.uuid',
-            'p.uuid AS parent_uuid', 'u.mtime',
-            'u.mtimetz',             'u.author',
-            'u.email',               'u.lang',
-            'u.message',             'u.ucount',
-        ],
-        from       => "$tmp t",
-        inner_join => 'updates u',
-        on         => 'u.id = t.id',
-        left_join  => 'updates p',
-        on         => 'p.id = u.parent_id',
-        order_by   => 'u.id ASC',
-    );
+        my $update_list = $db->xprepare(
+            select => [
+                'u.id',                  'u.uuid',
+                'p.uuid AS parent_uuid', 'u.mtime',
+                'u.mtimetz',             'u.author',
+                'u.email',               'u.lang',
+                'u.message',             'u.ucount',
+            ],
+            from       => "$tmp t",
+            inner_join => 'updates u',
+            on         => 'u.id = t.id',
+            left_join  => 'updates p',
+            on         => 'p.id = u.parent_id',
+            order_by   => 'u.id ASC',
+        );
 
-    $update_list->execute;
-    $self->send_updates($update_list) || return;
+        $update_list->execute;
+        return $self->send_updates( $update_list, $total );
+    };
 
     my ($uuid) = $db->xarray(
         select => 't.uuid',
@@ -198,7 +204,11 @@ sub real_sync_repo {
         where  => { 't.id' => $id },
     );
 
-    return $self->real_import_repo($uuid);
+    my $r1 = $self->real_import_repo($uuid);
+    my $r2 = $send->join;
+
+    return 'RepoSync' if ( $r1 eq 'RepoImported' and $r2 );
+    return $r1;
 }
 
 sub real_export_repo {
@@ -233,12 +243,9 @@ sub real_export_repo {
     );
 
     $sth->execute;
-    $self->send_updates($sth) || return;
+    $self->send_updates( $sth, $total ) || return;
 
-    $self->write( 'RepoExported', $total );
-    my ($action) = $self->read;
-    return 'RepoExported' if $action eq 'RepoImported';
-    return $action;
+    return 'RepoExported';
 }
 
 1;
